package hep.wired.heprep.graphicspanel;

import java.awt.Color;
import java.util.Properties;

import org.freehep.application.PropertyUtilities;
import org.freehep.jas.services.PreferencesTopic;

import hep.wired.heprep.plugin.WiredPlugin;
import hep.wired.heprep.services.GraphicsMode;
import hep.wired.plot.WiredPlot;
import hep.wired.util.SwingEventMulticaster;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DecimalFormat;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Defines the general graphics mode for each plot.
 * 
 * @author Mark Donszelmann
 * @version $Id: HepRepGraphicsMode.java 13945 2012-02-27 00:27:14Z onoprien $
 */
public final class HepRepGraphicsMode implements PreferencesTopic, GraphicsMode {
  
// -- Private parts : ----------------------------------------------------------

  static private final String _keyPrefix = "graphics.";
  
  static private final String _antiAliasKey = "antialias";
  static private final String _drawFramesKey = "drawframes";
  static private final String _fillBoxesKey = "fillboxes";
  static private final String _useLineWidthKey = "widelines";
  static private final String _lineWidthKey = "linewidth";
  static private final String _useLineStyleKey = "linestyle";
  static private final String _markSizeKey = "marksize";
  static private final String _useLayeringKey ="uselayering";
  static private final String _backColorKey = "backColor";
  static private final String _selectionModifyKey = "selectionmodify";
  static private final String _selectionChangeColorSelectedKey = "changecolorselected";
  static private final String _selectionColorSelectedKey = "colorselected";
  static private final String _selectionColorUnselectedKey = "colorunselected";
  static private final String _selectionLineWidthMultiplierKey = "selectionlinewidth";

  static private final boolean _antiAliasDefault = false;
  static private final boolean _drawFramesDefault = false;
  static private final boolean _fillBoxesDefault = false;
  static private final boolean _useLineWidthDefault = false;
  static private final double _lineWidthDefault = 1.;
  static private final boolean _useLineStyleDefault = false;
  static private final double _markSizeDefault = 1.;
  static private final boolean _useLayeringDefault = true;
  static private final Color _backColorDefault = Color.BLACK;
  static private final boolean _selectionModifyDefault = false;
  static private final boolean _selectionChangeColorSelectedDefault = true;
  static private final Color _selectionColorSelectedDefault = Color.MAGENTA;
  static private final Color _selectionColorUnselectedDefault = Color.GRAY;
  static private final double _selectionLineWidthMultiplierDefault = 1.;
  
  private Properties _properties; // Wired properties if this is default GraphicsMode, null if plot-specific GraphicsMode
  private HepRepPanel _plot; // graphics panel using this mode (null if this is default mode)
  
  private ChangeListener _listener;

  
// -- Properties accessible to clients : ---------------------------------------

  public boolean antiAlias;
  public boolean drawFrames;
  public boolean fillBoxes;
  public boolean useLineWidth;
  public double lineWidth;
  public boolean useLineStyle;
  public double markSizeMultiplier;
  public boolean useLayering;
  public Color backColor;
  public boolean selectionModify;
  public boolean selectionChangeColorSelected;
  public Color selectionColorSelected;
  public Color selectionColorUnselected;
  public double selectionLineWidthMultiplier;

  
// -- Construction and initialization : ----------------------------------------

  /** Creates <tt>HepRepGraphicsMode</tt> with default properties. */
  public HepRepGraphicsMode() {
    
    _properties = WiredPlugin.getPlugin().getProperties();
    
    antiAlias = PropertyUtilities.touchBoolean(_properties, _keyPrefix + _antiAliasKey, _antiAliasDefault);
    drawFrames = PropertyUtilities.touchBoolean(_properties, _keyPrefix + _drawFramesKey, _drawFramesDefault);
    fillBoxes = PropertyUtilities.touchBoolean(_properties, _keyPrefix + _fillBoxesKey, _fillBoxesDefault);
    useLineWidth = PropertyUtilities.touchBoolean(_properties, _keyPrefix + _useLineWidthKey, _useLineWidthDefault);
    lineWidth = PropertyUtilities.touchDouble(_properties, _keyPrefix + _lineWidthKey, _lineWidthDefault);
    useLineStyle = PropertyUtilities.touchBoolean(_properties, _keyPrefix + _useLineStyleKey, _useLineStyleDefault);
    markSizeMultiplier = PropertyUtilities.touchDouble(_properties, _keyPrefix + _markSizeKey, _markSizeDefault);
    useLayering = PropertyUtilities.touchBoolean(_properties, _keyPrefix + _useLayeringKey, _useLayeringDefault);
    backColor = PropertyUtilities.touchColor(_properties, _keyPrefix + _backColorKey, _backColorDefault);
    selectionModify = PropertyUtilities.touchBoolean(_properties, _keyPrefix + _selectionModifyKey, _selectionModifyDefault);
    selectionChangeColorSelected = PropertyUtilities.touchBoolean(_properties, _keyPrefix + _selectionChangeColorSelectedKey, _selectionChangeColorSelectedDefault);
    selectionColorSelected = PropertyUtilities.touchColor(_properties, _keyPrefix + _selectionColorSelectedKey, _selectionColorSelectedDefault);
    selectionColorUnselected = PropertyUtilities.touchColor(_properties, _keyPrefix + _selectionColorUnselectedKey, _selectionColorUnselectedDefault);
    selectionLineWidthMultiplier = PropertyUtilities.touchDouble(_properties, _keyPrefix + _selectionLineWidthMultiplierKey, _selectionLineWidthMultiplierDefault);
  }

  /** Creates <tt>HepRepGraphicsMode</tt> optimized for fast drawing. */
  public HepRepGraphicsMode(boolean fast) {
    
    Properties properties = WiredPlugin.getPlugin().getProperties();
    
    antiAlias = false;
    drawFrames = false;
    fillBoxes = false;
    useLineWidth = false;
    lineWidth = PropertyUtilities.getDouble(properties, _keyPrefix + _lineWidthKey, _lineWidthDefault);
    useLineStyle = false;
    markSizeMultiplier = PropertyUtilities.getDouble(properties, _keyPrefix + _markSizeKey, _markSizeDefault);
    useLayering = false;
    backColor = PropertyUtilities.getColor(properties, _keyPrefix + _backColorKey, _backColorDefault);
    selectionModify = PropertyUtilities.getBoolean(properties, _keyPrefix + _selectionModifyKey, _selectionModifyDefault);
    selectionChangeColorSelected = PropertyUtilities.touchBoolean(properties, _keyPrefix + _selectionChangeColorSelectedKey, _selectionChangeColorSelectedDefault);
    selectionColorSelected = PropertyUtilities.getColor(properties, _keyPrefix + _selectionColorSelectedKey, _selectionColorSelectedDefault);
    selectionColorUnselected = PropertyUtilities.getColor(properties, _keyPrefix + _selectionColorUnselectedKey, _selectionColorUnselectedDefault);
    selectionLineWidthMultiplier = PropertyUtilities.getDouble(properties, _keyPrefix + _selectionLineWidthMultiplierKey, _selectionLineWidthMultiplierDefault);
  }

  /** Creates a copy of the specified <tt>HepRepGraphicsMode</tt>, attached to the specified graphics panel. */
  public HepRepGraphicsMode(HepRepGraphicsMode mode, HepRepPanel plot) {
    
    _plot = plot;
    
    antiAlias = mode.antiAlias;
    drawFrames = mode.drawFrames;
    fillBoxes = mode.fillBoxes;
    useLineWidth = mode.useLineWidth;
    lineWidth = mode.lineWidth;
    useLineStyle = mode.useLineStyle;
    markSizeMultiplier = mode.markSizeMultiplier;
    useLayering = mode.useLayering;
    backColor = mode.backColor;
    selectionModify = mode.selectionModify;
    selectionChangeColorSelected = mode.selectionChangeColorSelected;
    selectionColorSelected = mode.selectionColorSelected;
    selectionColorUnselected = mode.selectionColorUnselected;
    selectionLineWidthMultiplier = mode.selectionLineWidthMultiplier;
  }


// -- Handling listeners : -----------------------------------------------------

  public void addChangeListener(ChangeListener listener) {
    _listener = SwingEventMulticaster.add(_listener, listener);
  }

  public void removeChangeListener(ChangeListener listener) {
    _listener = SwingEventMulticaster.remove(_listener, listener);
  }

  private void fireStateChanged() {
    if (_listener != null) {
      _listener.stateChanged(new ChangeEvent(this));
    }
  }
  
  
// -- Implementing PreferencesTopic : ------------------------------------------

  public boolean apply(JComponent jc) {

    boolean success = true;
    try {
      success = ((GUI) jc).apply();
    } catch (ClassCastException x) {
      return true;
    }
    
    if (success) {
      HepRepPanel panel = _plot;
      if (panel == null) {
        WiredPlot plot = WiredPlugin.getPlugin().getPlot();
        if (plot != null) {
          try {
            panel = (HepRepPanel) plot.getGraphicsPanel();
          } catch (ClassCastException x) {}
        }
      }
      if (panel != null) panel.repaint();
    }
    
    fireStateChanged();
    return success;
  }

  public JComponent component() {
    return new GUI();
  }

  public String[] path() {
    return new String[] {"WIRED 4", "Graphics"};
  }
  
  
// -- Implementing GraphicsMode (obsolete, re-implement or remove) : ----------- 

  public Object getValue(String name) {
    if (name.equals(_antiAliasKey)) {
      return antiAlias;
    } else if (name.equals(_drawFramesKey)) {
      return drawFrames;
    } else if (name.equals(_fillBoxesKey)) {
      return fillBoxes;
    } else if (name.equals(_useLineWidthKey)) {
      return useLineWidth;
    } else if (name.equals(_lineWidthKey)) {
      return lineWidth;
    } else if (name.equals(_useLineStyleKey)) {
      return useLineStyle;
    } else if (name.equals(_markSizeKey)) {
      return markSizeMultiplier;
    } else if (name.equals(_useLayeringKey)) {
      return useLayering;
    } else if (name.equals(_backColorKey)) {
      return backColor;
    } else if (name.equals(_selectionModifyKey)) {
      return selectionModify;
    } else if (name.equals(_selectionChangeColorSelectedKey)) {
      return selectionChangeColorSelected;
    } else if (name.equals(_selectionColorSelectedKey)) {
      return selectionColorSelected;
    } else if (name.equals(_selectionColorUnselectedKey)) {
      return selectionColorUnselected;
    } else if (name.equals(_selectionLineWidthMultiplierKey)) {
      return selectionLineWidthMultiplier;
    } else {
      return null;
    }
  }

  public void setValue(String name, Object value) {
    if (name.equals(_antiAliasKey)) {
      antiAlias = value == null ? _antiAliasDefault : (Boolean) value;
      if (_properties != null) {
        PropertyUtilities.setBoolean(_properties, _keyPrefix + name, antiAlias);
      }
    } else if (name.equals(_drawFramesKey)) {
      drawFrames = value == null ? _drawFramesDefault : (Boolean) value;
      if (_properties != null) {
        PropertyUtilities.setBoolean(_properties, _keyPrefix + name, drawFrames);
      }
    } else if (name.equals(_fillBoxesKey)) {
      fillBoxes = value == null ? _fillBoxesDefault : (Boolean) value;
      if (_properties != null) {
        PropertyUtilities.setBoolean(_properties, _keyPrefix + name, fillBoxes);
      }
    } else if (name.equals(_useLineWidthKey)) {
      useLineWidth = value == null ? _useLineWidthDefault : (Boolean) value;
      if (_properties != null) {
        PropertyUtilities.setBoolean(_properties, _keyPrefix + name, useLineWidth);
      }
    } else if (name.equals(_lineWidthKey)) {
      lineWidth = value == null ? _lineWidthDefault : (Double) value;
      if (_properties != null) {
        PropertyUtilities.setDouble(_properties, _keyPrefix + name, lineWidth);
      }
    } else if (name.equals(_useLineStyleKey)) {
      useLineStyle = value == null ? _useLineStyleDefault : (Boolean) value;
      if (_properties != null) {
        PropertyUtilities.setBoolean(_properties, _keyPrefix + name, useLineStyle);
      }
    } else if (name.equals(_markSizeKey)) {
      markSizeMultiplier = value == null ? _markSizeDefault : (Double) value;
      if (_properties != null) {
        PropertyUtilities.setDouble(_properties, _keyPrefix + name, markSizeMultiplier);
      }
    } else if (name.equals(_useLayeringKey)) {
      useLayering = value == null ? _useLayeringDefault : (Boolean) value;
      if (_properties != null) {
        PropertyUtilities.setBoolean(_properties, _keyPrefix + name, useLayering);
      }
    } else if (name.equals(_backColorKey)) {
      backColor = (Color) value;
      if (_properties != null) {
        PropertyUtilities.setColor(_properties, _keyPrefix + name, backColor);
      }
    } else if (name.equals(_selectionModifyKey)) {
      selectionModify = value == null ? _selectionModifyDefault : (Boolean) value;
      if (_properties != null) {
        PropertyUtilities.setBoolean(_properties, _keyPrefix + name, selectionModify);
      }
    } else if (name.equals(_selectionChangeColorSelectedKey)) {
      selectionChangeColorSelected = (Boolean) value;
      if (_properties != null) {
        PropertyUtilities.setBoolean(_properties, _keyPrefix + name, selectionChangeColorSelected);
      }
    } else if (name.equals(_selectionColorSelectedKey)) {
      selectionColorSelected = (Color) value;
      if (_properties != null) {
        PropertyUtilities.setColor(_properties, _keyPrefix + name, selectionColorSelected);
      }
    } else if (name.equals(_selectionColorUnselectedKey)) {
      selectionColorUnselected = (Color) value;
      if (_properties != null) {
        PropertyUtilities.setColor(_properties, _keyPrefix + name, selectionColorUnselected);
      }
    } else if (name.equals(_selectionLineWidthMultiplierKey)) {
      selectionLineWidthMultiplier = value == null ? _selectionLineWidthMultiplierDefault : (Double) value;
      if (_properties != null) {
        PropertyUtilities.setDouble(_properties, _keyPrefix + name, selectionLineWidthMultiplier);
      }
    }
  }
  
  
// -- Override Object : --------------------------------------------------------
  
  /**
   * Returns <tt>true</tt> if settings in the specified graphics mode object are identical to the settings in this object.
   */
  public boolean isSame(HepRepGraphicsMode other) {
    return antiAlias == other.antiAlias &&
           drawFrames == other.drawFrames &&
           fillBoxes == other.fillBoxes &&
           useLineWidth == other.useLineWidth &&
           Math.abs(lineWidth - other.lineWidth) < .01 &&
           useLineStyle == other.useLineStyle &&
           Math.abs(markSizeMultiplier - other.markSizeMultiplier) < .01 &&
           useLayering == other.useLayering &&
           backColor.equals(other.backColor) &&
           selectionModify == other.selectionModify &&
           selectionChangeColorSelected == other.selectionChangeColorSelected &&
           selectionColorSelected.equals(other.selectionColorSelected) &&
           selectionColorUnselected.equals(other.selectionColorUnselected) &&
           Math.abs(selectionLineWidthMultiplier - other.selectionLineWidthMultiplier) < .01;
  }
  
  
// -- GUI : --------------------------------------------------------------------
  
  private class GUI extends JPanel {
    
    private JCheckBox _antiAliasBox;
    private JCheckBox _drawFramesBox;
    private JCheckBox _drawWideLinesBox;
    private JCheckBox _fillBoxesBox;
    private JFormattedTextField _lineWidthField;
    private JLabel _lineWidthLabel;
    private JCheckBox _useLineStyleBox;
    private JFormattedTextField _markSizeField;
    private JCheckBox _useLayeringBox;
    private JButton _backColorButton;
    private JRadioButton _selectionModifiedRadio;
    private JCheckBox _selectionChangeColorSelectedBox;
    private JButton _selectionColorButtonSelected;
    private JFormattedTextField _selectionLineWidthField;
    private JRadioButton _selectionUnmodifiedRadio;
    private JButton _selectionColorButtonUnselected;
    
    private int HSPACE1 = 5;
    private int VSPACE1 = 5;
    

    GUI() {
      setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
      setBorder(BorderFactory.createEmptyBorder(4, 4, 2, 2));

      _antiAliasBox = new JCheckBox("Anti-alias");
      add(_antiAliasBox);

      _drawFramesBox = new JCheckBox("Draw frames");
      add(_drawFramesBox);

      _fillBoxesBox = new JCheckBox("Fill boxes");
      add(_fillBoxesBox);

      Box box = Box.createHorizontalBox();
      box.setAlignmentX(Component.LEFT_ALIGNMENT);
      box.add(Box.createRigidArea(new Dimension(HSPACE1, 0)));
      _lineWidthLabel = new JLabel("Line width ");
      box.add(_lineWidthLabel);
      box.add(Box.createRigidArea(new Dimension(HSPACE1, 0)));
      _lineWidthField = new JFormattedTextField(new DecimalFormat("#0.0#")) {
        public Dimension getMaximumSize() {
          return getPreferredSize();
        }
      };
      _lineWidthField.setFocusLostBehavior(JFormattedTextField.COMMIT_OR_REVERT);
      _lineWidthField.setColumns(6);
      _lineWidthField.setHorizontalAlignment(JTextField.RIGHT);
      box.setAlignmentX(Component.LEFT_ALIGNMENT);
      box.add(_lineWidthField);
      add(box);
      
      _drawWideLinesBox = new JCheckBox("Honor line width attributes ");
      add(_drawWideLinesBox);
      
      _useLineStyleBox = new JCheckBox("Honor line style attributes (slow) ");
      add(_useLineStyleBox);

      box = Box.createHorizontalBox();
      box.setAlignmentX(LEFT_ALIGNMENT);
      box.add(new JLabel("Mark size multiplier"));
      box.add(Box.createRigidArea(new Dimension(HSPACE1, 0)));
      _markSizeField = new JFormattedTextField(new DecimalFormat("#0.0#")) {
        public Dimension getMaximumSize() {
          return getPreferredSize();
        }
      };
      _markSizeField.setFocusLostBehavior(JFormattedTextField.COMMIT_OR_REVERT);
      _markSizeField.setColumns(6);
      _markSizeField.setHorizontalAlignment(JTextField.RIGHT);
      box.add(_markSizeField);
      add(box);
      
      add(Box.createRigidArea(new Dimension(0, VSPACE1)));
      add(new JSeparator());
      add(Box.createRigidArea(new Dimension(0, VSPACE1)));

      _useLayeringBox = new JCheckBox("Use layering");
      add(_useLayeringBox);
      
      add(Box.createRigidArea(new Dimension(0, VSPACE1)));
      add(new JSeparator());
      add(Box.createRigidArea(new Dimension(0, VSPACE1)));
     
      box = Box.createHorizontalBox();
      box.setAlignmentX(LEFT_ALIGNMENT);
      box.add(new JLabel("Background color"));
      box.add(Box.createRigidArea(new Dimension(HSPACE1, 0)));
      _backColorButton = new JButton(" ");
      _backColorButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          Color color = JColorChooser.showDialog(GUI.this, "", _backColorButton.getBackground());
          if (color != null) _backColorButton.setBackground(color);
        }
      });
      box.add(_backColorButton);
      add(box);
      
      add(Box.createRigidArea(new Dimension(0, VSPACE1)));
      
      _selectionModifiedRadio = new JRadioButton();
      _selectionModifiedRadio.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          _selectionChangeColorSelectedBox.setEnabled(true);
          _selectionColorButtonSelected.setEnabled(true);
          _selectionLineWidthField.setEnabled(true);
          _selectionColorButtonUnselected.setEnabled(false);
        }
      });
      _selectionUnmodifiedRadio = new JRadioButton();
      _selectionUnmodifiedRadio.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          _selectionChangeColorSelectedBox.setEnabled(false);
          _selectionColorButtonSelected.setEnabled(false);
          _selectionLineWidthField.setEnabled(false);
          _selectionColorButtonUnselected.setEnabled(true);
        }
      });
      ButtonGroup selectionButtonGroup = new ButtonGroup();
      selectionButtonGroup.add(_selectionModifiedRadio);
      selectionButtonGroup.add(_selectionUnmodifiedRadio);
      _selectionChangeColorSelectedBox = new JCheckBox("Color");
      _selectionChangeColorSelectedBox.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          _selectionColorButtonSelected.setEnabled(_selectionChangeColorSelectedBox.isSelected());
        }
      });
      _selectionColorButtonSelected = new JButton(" ");
      _selectionColorButtonSelected.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          Color color = JColorChooser.showDialog(GUI.this, "", _selectionColorButtonSelected.getBackground());
          if (color != null) _selectionColorButtonSelected.setBackground(color);
        }
      });
      _selectionColorButtonUnselected = new JButton(" ");
      _selectionColorButtonUnselected.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          Color color = JColorChooser.showDialog(GUI.this, "", _selectionColorButtonUnselected.getBackground());
          if (color != null) _selectionColorButtonUnselected.setBackground(color);
        }
      });
      _selectionLineWidthField = new JFormattedTextField(new DecimalFormat("#0.0#")) {
        public Dimension getMaximumSize() {
          return getPreferredSize();
        }
      };
      _selectionLineWidthField.setFocusLostBehavior(JFormattedTextField.COMMIT_OR_REVERT);
      _selectionLineWidthField.setColumns(6);
      _selectionLineWidthField.setHorizontalAlignment(JTextField.RIGHT);
      Box selectionBox = Box.createVerticalBox();
      selectionBox.setBorder(BorderFactory.createTitledBorder("Selection display"));
      selectionBox.setAlignmentX(LEFT_ALIGNMENT);
      box = Box.createHorizontalBox();
      box.add(_selectionModifiedRadio);
      box.add(Box.createRigidArea(new Dimension(HSPACE1, 0)));
      box.add(new JLabel("Selected objects: "));
      box.add(Box.createRigidArea(new Dimension(HSPACE1, 0)));
      box.add(_selectionChangeColorSelectedBox);
      box.add(Box.createRigidArea(new Dimension(HSPACE1, 0)));
      box.add(_selectionColorButtonSelected);
      box.add(Box.createRigidArea(new Dimension(HSPACE1, 0)));
      box.add(new JLabel("Line width factor:"));
      box.add(Box.createRigidArea(new Dimension(HSPACE1, 0)));
      box.add(_selectionLineWidthField);
      box.add(Box.createHorizontalGlue());
      selectionBox.add(box);
      selectionBox.add(Box.createRigidArea(new Dimension(0, VSPACE1)));
      box = Box.createHorizontalBox();
      box.add(_selectionUnmodifiedRadio);
      box.add(Box.createRigidArea(new Dimension(HSPACE1, 0)));
      box.add(new JLabel("Not selected objects: "));
      box.add(Box.createRigidArea(new Dimension(HSPACE1, 0)));
      box.add(_selectionColorButtonUnselected);
      box.add(Box.createRigidArea(new Dimension(HSPACE1, 0)));
      box.add(Box.createHorizontalGlue());
      selectionBox.add(box);
      add(selectionBox);
      
      add(Box.createRigidArea(new Dimension(0, 2*VSPACE1)));
     
      JButton resetButton = new JButton("Reset");
      resetButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          reset();
        }
      });
      add(resetButton);
      add(Box.createRigidArea(new Dimension(0, VSPACE1)));

      add(Box.createVerticalGlue());
      
      _antiAliasBox.setSelected(antiAlias);
      _drawFramesBox.setSelected(drawFrames);
      _drawWideLinesBox.setSelected(useLineWidth);
      _fillBoxesBox.setSelected(fillBoxes);
      _lineWidthField.setValue((Double) lineWidth);
      _markSizeField.setValue((Double) markSizeMultiplier);
      _useLayeringBox.setSelected(useLayering);
      _backColorButton.setBackground(backColor);
      _selectionChangeColorSelectedBox.setSelected(selectionChangeColorSelected);
      _selectionColorButtonSelected.setBackground(selectionColorSelected);
      _selectionColorButtonUnselected.setBackground(selectionColorUnselected);
      _selectionLineWidthField.setValue((Double) selectionLineWidthMultiplier);
      if (selectionModify) {
        _selectionModifiedRadio.doClick();
      } else {
        _selectionUnmodifiedRadio.doClick();
      }
      
    }
    
    boolean apply() {
      
      antiAlias = _antiAliasBox.isSelected();
      drawFrames = _drawFramesBox.isSelected();
      useLineWidth = _drawWideLinesBox.isSelected();
      fillBoxes = _fillBoxesBox.isSelected();
      lineWidth = getDouble(_lineWidthField.getValue(), _lineWidthDefault);
      useLineStyle = _useLineStyleBox.isSelected();
      markSizeMultiplier = getDouble(_markSizeField.getValue(), _markSizeDefault);
      useLayering = _useLayeringBox.isSelected();
      backColor = _backColorButton.getBackground();
      selectionModify = _selectionModifiedRadio.isSelected();
      selectionChangeColorSelected = _selectionChangeColorSelectedBox.isSelected();
      selectionColorSelected = _selectionColorButtonSelected.getBackground();
      selectionColorUnselected = _selectionColorButtonUnselected.getBackground();
      selectionLineWidthMultiplier = getDouble(_selectionLineWidthField.getValue(), _selectionLineWidthMultiplierDefault);
      
      if (_properties != null) {
        PropertyUtilities.setBoolean(_properties, _keyPrefix + _antiAliasKey, antiAlias);
        PropertyUtilities.setBoolean(_properties, _keyPrefix + _drawFramesKey, drawFrames);
        PropertyUtilities.setBoolean(_properties, _keyPrefix + _fillBoxesKey, fillBoxes);
        PropertyUtilities.setBoolean(_properties, _keyPrefix + _useLineWidthKey, useLineWidth);
        PropertyUtilities.setDouble(_properties, _keyPrefix + _lineWidthKey, lineWidth);
        PropertyUtilities.setBoolean(_properties, _keyPrefix + _useLineStyleKey, useLineStyle);
        PropertyUtilities.setDouble(_properties, _keyPrefix + _markSizeKey, markSizeMultiplier);
        PropertyUtilities.setBoolean(_properties, _keyPrefix + _useLayeringKey, useLayering);
        PropertyUtilities.setColor(_properties, _keyPrefix + _backColorKey, backColor);
        PropertyUtilities.setBoolean(_properties, _keyPrefix + _selectionModifyKey, selectionModify);
        PropertyUtilities.setBoolean(_properties, _keyPrefix + _selectionChangeColorSelectedKey, selectionChangeColorSelected);
        PropertyUtilities.setColor(_properties, _keyPrefix + _selectionColorSelectedKey, selectionColorSelected);
        PropertyUtilities.setColor(_properties, _keyPrefix + _selectionColorUnselectedKey, selectionColorUnselected);
        PropertyUtilities.setDouble(_properties, _keyPrefix + _selectionLineWidthMultiplierKey, selectionLineWidthMultiplier);
      }
      
      return true;
    }
    
    /**
     * If this is the default GraphicsMode, reset gui to default values.
     * If this is plot-specific GraphicsMode, reset gui to current settings of the default GraphicsMode.
     */
    private void reset() {
      if (_plot == null) {
        _antiAliasBox.setSelected(_antiAliasDefault);
        _drawFramesBox.setSelected(_drawFramesDefault);
        _drawWideLinesBox.setSelected(_useLineWidthDefault);
        _fillBoxesBox.setSelected(_fillBoxesDefault);
        _lineWidthField.setValue((Double) _lineWidthDefault);
        _useLineStyleBox.setSelected(_useLineStyleDefault);
        _markSizeField.setValue((Double) _markSizeDefault);
        _useLayeringBox.setSelected(_useLayeringDefault);
        _backColorButton.setBackground(_backColorDefault);
        _selectionChangeColorSelectedBox.setSelected(_selectionChangeColorSelectedDefault);
        _selectionColorButtonSelected.setBackground(_selectionColorSelectedDefault);
        _selectionColorButtonUnselected.setBackground(_selectionColorUnselectedDefault);
        _selectionLineWidthField.setValue((Double) _selectionLineWidthMultiplierDefault);
        if (_selectionModifyDefault) {
          _selectionModifiedRadio.doClick();
        } else {
          _selectionUnmodifiedRadio.doClick();
        }
      } else {
        HepRepGraphicsMode defaultMode = WiredPlugin.getPlugin().getGraphicsMode();
        _antiAliasBox.setSelected(defaultMode.antiAlias);
        _drawFramesBox.setSelected(defaultMode.drawFrames);
        _drawWideLinesBox.setSelected(defaultMode.useLineWidth);
        _fillBoxesBox.setSelected(defaultMode.fillBoxes);
        _lineWidthField.setValue((Double) defaultMode.lineWidth);
        _useLineStyleBox.setSelected(defaultMode.useLineStyle);
        _markSizeField.setValue((Double) defaultMode.markSizeMultiplier);
        _useLayeringBox.setSelected(defaultMode.useLayering);
        _backColorButton.setBackground(defaultMode.backColor);
        _selectionChangeColorSelectedBox.setSelected(defaultMode.selectionChangeColorSelected);
        _selectionColorButtonSelected.setBackground(defaultMode.selectionColorSelected);
        _selectionColorButtonUnselected.setBackground(defaultMode.selectionColorUnselected);
        _selectionLineWidthField.setValue((Double) defaultMode.selectionLineWidthMultiplier);
        if (defaultMode.selectionModify) {
          _selectionModifiedRadio.doClick();
        } else {
          _selectionUnmodifiedRadio.doClick();
        }
      }
    }
    
    /** Converts object returned by JFormattedTextField's getValue() method into double. */
    private double getDouble(Object value, double defaultValue) {
      try {
        return ((Double) value).doubleValue();
      } catch (ClassCastException x) {
        try {
          return ((Long) value).doubleValue();
        } catch (ClassCastException xx) {
          return defaultValue;
        }
      }
    }
    
  }

// -----------------------------------------------------------------------------
}
