package hep.wired.heprep.interaction;

import hep.graphics.heprep.HepRepAttDef;
import hep.graphics.heprep.HepRepAttValue;
import hep.graphics.heprep.HepRepInstance;
import hep.graphics.heprep.HepRepPoint;
import hep.graphics.heprep.HepRepType;
import hep.graphics.heprep.util.HepRepUtil;
import hep.wired.util.SortableListTableModel;

import java.util.*;

/**
 * TableModel for HepRep Attributes.
 * 
 * @author Mark Donszelmann
 * @author Dmitry Onoprienko
 * @version $Id: AttributeTableModel.java 13931 2012-02-09 00:58:03Z onoprien $
 */
public class AttributeTableModel extends SortableListTableModel {

// -- Private parts : ----------------------------------------------------------

  private Set<String> _shownCategorySet;

  private static final int ATTNAME = 0;
  private static final int ATTVALUE = 1;
  private static final int ATTUNIT = 2;
  private static final int ATTNODE = 3;

  private HepRepInstance _instance;

  private Set<String> attributeNames = new HashSet();
  private List<HepRepAttValue> attributeValues;
  private Map<HepRepAttValue, String> attributeNodes = new HashMap();

// -- Construction : -----------------------------------------------------------

  // May be null if no categories exist
  public AttributeTableModel(List<Flag> categories) {
    super(new String[]{"Name", "Value", "Unit", "Node"}, new Class[]{
          String.class, Class.class, String.class, String.class},
          new ArrayList());
    attributeValues = getKeys();
    if (categories == null) {
      _shownCategorySet = null;
    } else {
      _shownCategorySet = new HashSet();
      for (Flag f : categories) {
        if (f.isSet()) _shownCategorySet.add(f.getName());
      }
    }
  }

// -- Setters : ----------------------------------------------------------------

  public void setInstance(HepRepInstance instance) {
    _instance = instance;
  }

  public void setShownCategories(List<Flag> categories) {
    _shownCategorySet = new HashSet();
    for (Flag f : categories) {
      if (f.isSet()) _shownCategorySet.add(f.getName());
    }
    if (categories.size() ==  _shownCategorySet.size())  _shownCategorySet = null;
  }

// -- Updating model : ---------------------------------------------------------

  public void update() {
    attributeNames.clear();
    attributeValues.clear();
    attributeNodes.clear();

    HepRepInstance i = _instance;
    boolean onNode = true;
    while (i != null) {
      // add the attributes of the instance
      HepRepType type = i.getType();
      addAttributeValues(type, i.getAttValuesFromNode(), onNode ? "Instance" : "Parent");

      // add the attributes of the points
      if (onNode) {
        Iterator j = i.getPoints().iterator();
        int n = 1;
        while (j.hasNext()) {
          HepRepPoint p = (HepRepPoint) j.next();
          addAttributeValues(type, p.getAttValuesFromNode(), "Point" + n);
          n++;
        }
      }

      // add the attributes of the instance's type (and the type's
      // parents)
      while (type != null) {
        addAttributeValues(type, type.getAttValuesFromNode(), "Type");
        type = type.getSuperType();
      }

      // go to the instance's parent
      if (i.getAttValue("ShowParentAttributes").getBoolean()) {
        i = i.getSuperInstance();
        onNode = false;
      } else {
        i = null;
      }
    }
    fireTableDataChanged();
  }

  private void addAttributeValues(HepRepType type, Set values, String node) {
    for (Iterator i = values.iterator(); i.hasNext();) {
      HepRepAttValue value = (HepRepAttValue) i.next();
      String name = value.getLowerCaseName();

      // duplicates are allowed for atts on points.
      if (node.startsWith("Point") || !attributeNames.contains(name)) {
        HepRepAttDef attDef = type.getAttDef(value.getName());
        if (attDef == null || _shownCategorySet == null || _shownCategorySet.contains(attDef.getCategory())) {
          attributeNames.add(name);
          attributeValues.add(value);
          attributeNodes.put(value, node);
        }
      }
    }
  }

// -- Implementing table model : -----------------------------------------------

  public int getRowCount() {
    return (_instance != null) ? attributeValues.size() : 0;
  }

  public Object getValueAt(int row, int col) {
    HepRepAttValue value = (HepRepAttValue) attributeValues.get(row);
    switch (col) {
      case ATTNAME:
        return value.getName();
      case ATTVALUE:
        switch (value.getType()) {
          case HepRepAttValue.TYPE_BOOLEAN:
            return (Boolean) value.getBoolean();
          case HepRepAttValue.TYPE_COLOR:
            return value.getColor();
          case HepRepAttValue.TYPE_DOUBLE:
            return new Double(value.getDouble());
          case HepRepAttValue.TYPE_INT:
            return new Integer(value.getInteger());
          case HepRepAttValue.TYPE_LONG:
            return new Long(value.getLong());
          case HepRepAttValue.TYPE_STRING:
            return value.getString();
          case HepRepAttValue.TYPE_UNKNOWN:
          default:
            return value.getAsString();
        }
      case ATTUNIT:
        HepRepAttDef def = _instance.getType().getAttDef(value.getName());
        return def != null ? HepRepUtil.getUnit(def) : "";
      case ATTNODE:
        return (String) attributeNodes.get(value);
      default:
        return "";
    }
  }

  protected int compareKeys(Object o1, Object o2, int col, boolean ascending) {
    HepRepAttValue a1 = (HepRepAttValue) o1;
    HepRepAttValue a2 = (HepRepAttValue) o2;
    switch (col) {
      default:
      case ATTNAME:
        return (ascending ? 1 : -1)
                * (a1.getName().compareTo(a2.getName()));
      case ATTVALUE:
        if (a1.getType() == a2.getType()) {
          switch (a1.getType()) {
            case HepRepAttValue.TYPE_BOOLEAN:
              return (ascending ? 1 : -1)
                      * (a1.getBoolean() ? (a2.getBoolean() ? 0 : 1) : a2.getBoolean() ? -1 : 0);

            case HepRepAttValue.TYPE_COLOR: // No order...
              return 0;

            case HepRepAttValue.TYPE_DOUBLE:
              return (ascending ? 1 : -1)
                      * (a1.getDouble() < a2.getDouble() ? -1 : a1.getDouble() > a2.getDouble() ? 1 : 0);

            case HepRepAttValue.TYPE_INT:
              return (ascending ? 1 : -1)
                      * (a1.getInteger() < a2.getInteger() ? -1 : a1.getInteger() > a2.getInteger() ? 1 : 0);

            case HepRepAttValue.TYPE_LONG:
              return (ascending ? 1 : -1)
                      * (a1.getLong() < a2.getLong() ? -1
                      : a1.getLong() > a2.getLong() ? 1 : 0);

            case HepRepAttValue.TYPE_STRING:
              return (ascending ? 1 : -1)
                      * (a1.getString().compareTo(a2.getString()));

            case HepRepAttValue.TYPE_UNKNOWN:
            default:
              return (ascending ? 1 : -1)
                      * (a1.getAsString().compareTo(a2.getAsString()));
          }
        } else {
          return (ascending ? 1 : -1)
                  * (a1.getType() < a2.getType() ? -1 : 1);
        }
      case ATTUNIT:
        HepRepAttDef d1 = _instance.getType().getAttDef(a1.getName());
        HepRepAttDef d2 = _instance.getType().getAttDef(a2.getName());
        String u1 = d1 != null ? HepRepUtil.getUnit(d1) : "";
        String u2 = d2 != null ? HepRepUtil.getUnit(d2) : "";
        return (ascending ? 1 : -1) * (u1.compareTo(u2));
      case ATTNODE:
        String b1 = (String) attributeNodes.get(a1);
        String b2 = (String) attributeNodes.get(a2);
        return (ascending ? 1 : -1) * (b1.compareTo(b2));
    }
  }
}
