// Copyright 2006, FreeHEP.
package hep.wired.heprep.interaction;

import hep.wired.heprep.image.WiredImage;
import hep.wired.heprep.services.InverseProjection;
import hep.wired.services.GraphicsPanel;
import hep.wired.services.RecordPlot;
import hep.wired.services.ViewPort;
import hep.wired.util.XYZindices;

import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;

import javax.swing.Icon;

import org.freehep.application.Application;
import org.freehep.util.ScientificFormat;

/**
 * Drag/click to measure distances in 2D on the plot.
 * 
 * @author Mark Donszelmann
 * @version $Id: DragToMeasure.java 14093 2013-01-09 21:57:40Z onoprien $
 */
public class DragToMeasure extends DefaultWiredInteractionHandler implements XYZindices {

    private int xp, yp;

    private int xd, yd;
    
    private Line2D line;

    /**
     * Create a rectangle handler to generate scale edits.
     */
    public DragToMeasure() {
        super("Drag to Measure");
    }

    public Icon getIcon(int size) {
        return WiredImage.getIcon("Ruler%w", size);
    }

    public String getDescription() {
        return "Drag line to measure distance between endpoints, Shift forces horizontal/vertical measure.";
    }

    public boolean isSupportedBy(GraphicsPanel panel) {
        return panel.supports(InverseProjection.class);
    }    
    
    public void changeCursor(RecordPlot plot, InputEvent event) {
        plot.setCursor(null);
    }

    public void reset(RecordPlot plot, InputEvent event) {
        plot.drawShape(null);
        Application.getApplication().setStatusMessage(getDescription());
    }

    public void mouseEntered(RecordPlot plot, MouseEvent event) {
        plot.requestFocusInWindow();
        changeCursor(plot, event);
    }

    public void mouseWheelMoved(RecordPlot plot, MouseWheelEvent event) {
        super.mouseWheelMoved(plot, event);
        defineLine(plot, event);
        measure(plot, line);        
    }
    
    /**
     * Store starting point of line.
     */
    public void mouseButton1DragStarted(RecordPlot plot, MouseEvent event) {
        xp = event.getX();
        yp = event.getY();
        changeCursor(plot, event);
        reset(plot, event);
    }

    /**
     * Update moving point of line and draw.
     */
    public void mouseButton1Dragged(RecordPlot plot, MouseEvent event) {
        xd = event.getX();
        yd = event.getY();
        defineLine(plot, event);
        measure(plot, line);
    }

    /**
     * Update end point of line, translate and scale, and remove rectangle.
     */
    public void mouseButton1DragEnded(RecordPlot plot, MouseEvent event) {
        measure(plot, line);
    }

    public boolean shiftKeyPressed(RecordPlot plot, KeyEvent event) {
        defineLine(plot, event);
        return true;
    }

    public boolean shiftKeyReleased(RecordPlot plot, KeyEvent event) {
        defineLine(plot, event);
        return true;
    }

    private void defineLine(RecordPlot plot, InputEvent event) {
        // no need to define if we pressed a key and rectangle was defined
        // before
        if ((event instanceof KeyEvent) && (line == null))
            return;

        int dx = xd - xp;
        int dy = yd - yp;

        // restrict line to vertical/horizontal when shift is pressed
        if (event.isShiftDown()) {
            if (Math.abs(dy) < Math.abs(dx)) {
                dy = 0;
            } else {
                dx = 0;
            }
        }
        line = new Line2D.Double(xp, yp, xp + dx, yp + dy);
        
        // perpendicular angle
        double d = 6;
        double a = Math.atan2(dx, dy);
        double cosa = d*Math.cos(a);
        double sina = d*Math.sin(a);
        
        GeneralPath path = new GeneralPath();
        path.append(line, false);
        path.append(new Line2D.Double(xp+cosa, yp-sina, xp-cosa, yp+sina), false);
        path.append(new Line2D.Double(xp+dx+cosa, yp+dy-sina, xp+dx-cosa, yp+dy+sina), false);
        plot.drawShape(path);
    }

    private void measure(RecordPlot plot, Line2D line) {
        try {
            ViewPort viewPort = plot.getGraphicsPanel().getViewPort();
            InverseProjection projection = (InverseProjection)plot.getGraphicsPanel().getFeature(InverseProjection.class);
            double p1[] = new double[] { line.getX1(), line.getY1(), 0 };
            double p2[] = new double[] { line.getX2(), line.getY2(), 0 };
            p1 = viewPort.inverseTransform(p1);
            p2 = viewPort.inverseTransform(p2);
            p1 = projection.inverseTransform(p1);
            p2 = projection.inverseTransform(p2);
            double dx = p2[X] - p1[X];
            double dy = p2[Y] - p1[Y];
            double dz = p2[Z] - p1[Z];
            double d = Math.sqrt(dx*dx+dy*dy+dz*dz);
            Application.getApplication().setStatusMessage("Distance: "+_format.format(d));
        } catch (UnsupportedOperationException e) {
            Application.getApplication().setStatusMessage("Distance cannot be calculated.");
        }
    }
    
    public String toString() {
        return "Drag to Measure";
    }
}
