// Copyright 2005-2006, FreeHEP.
package hep.wired.heprep.interaction;

import hep.wired.heprep.graphicspanel.ControlPanel;
import hep.wired.heprep.image.WiredImage;
import hep.wired.heprep.plugin.WiredPlugin;
import hep.wired.util.ScientificTable;
import hep.wired.util.SwingEventMulticaster;
import hep.wired.util.TitledPanel;
import hep.wired.variable.BooleanVariable;
import hep.wired.variable.DoubleVariable;
import hep.wired.variable.DoubleVariableSliderModel;
import hep.wired.variable.IntegerVariable;
import hep.wired.variable.IntegerVariableSliderModel;
import hep.wired.variable.Variable;
import hep.wired.variable.VariableChangeEvent;
import hep.wired.variable.VariableSlider;
import hep.wired.variable.VariableTableModel;

import java.awt.Component;
import java.awt.Font;
import java.text.Format;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableCellRenderer;

import org.freehep.swing.layout.TableLayout;
import org.freehep.util.ScientificFormat;

/**
 * VariablePanel.
 * 
 * @author Mark Donszelmann
 * @version $Id: VariablePanel.java 13827 2011-05-05 03:08:47Z onoprien $
 */
public class VariablePanel extends ControlPanel implements ChangeListener {

    private final String insets = " [3 3 3 3] ";

    private List variableList = Collections.EMPTY_LIST;

    private ChangeListener listener;

    private int selectedIndex = -1;

    private VariableTableModel variableTableModel;

    private ScientificTable variableTable;

    private JPanel variablePanel;

    private VariableSlider slider;

    private DoubleVariableSliderModel doubleSliderModel;

    private IntegerVariableSliderModel integerSliderModel;

    public VariablePanel() {
        super("Settings", "Set or Change the values of variables", WiredImage.getIcon("Settings%w", 16), WiredPlugin.IH.TRANSLATE.get());
        setLayout(new TableLayout());

        variableTableModel = new VariableTableModel();
        variableTable = new ScientificTable(variableTableModel) {

            // override tooltip to be description of the variable
            public Component prepareRenderer(TableCellRenderer renderer,
                    int rowIndex, int vColIndex) {
                Component c = super.prepareRenderer(renderer, rowIndex,
                        vColIndex);
                if (c instanceof JComponent) {
                    Variable v = (Variable) variableList.get(rowIndex);
                    if (v != null)
                        ((JComponent) c).setToolTipText(v.getDescription());

                    if (rowIndex == selectedIndex) {
                        c.setFont(c.getFont().deriveFont(Font.BOLD));
                    }

                    c.setEnabled(v.isEnabled());
                }
                return c;
            }
        };
        variableTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        variablePanel = new TitledPanel("Choose:", variableTable);

        doubleSliderModel = new DoubleVariableSliderModel();
        integerSliderModel = new IntegerVariableSliderModel();

        slider = new VariableSlider(doubleSliderModel);
        slider.setEnabled(true);

        add(variablePanel, "0 0" + insets + "wh");
        add(slider, "1 0" + insets + "h");
        
        // hook selection from table to slider
        variableTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent event) {
                if (event.getValueIsAdjusting())
                    return;

                if (variableTable.getRowCount() <= 0) {
                    setSlider(-1);
                    return;
                } 
                
                // only set if in table (may loose focus, keep old selection)
                int selectedRow = variableTable.getSelectedRow();
                if (selectedRow >= 0) setSlider(selectedRow);
            }
        });

        // initial List
        setVariables(Collections.EMPTY_LIST);
    }

    private void setSlider(int index) {
        selectedIndex = index;
        if ((index < 0) || (index >= variableList.size())) {
            slider.setModel(null);
            return;
        }
        Variable variable = (Variable) variableList.get(index);
        if (variable instanceof DoubleVariable) {
            doubleSliderModel.setVariable((DoubleVariable) variable);
            slider.setModel(doubleSliderModel);

        } else if (variable instanceof IntegerVariable) {
            integerSliderModel.setVariable((IntegerVariable) variable);
            slider.setModel(integerSliderModel);

        } else if (variable instanceof BooleanVariable) {
            slider.setModel(null);

        } else {
            throw new IllegalArgumentException(
                    getClass()
                            + ": ListSelectionListener cannot handle Variables of class: "
                            + variable.getClass());
        }
        revalidate();
        repaint();
    }

    public void addChangeListener(ChangeListener a) {
        listener = SwingEventMulticaster.add(listener, a);
    }

    public void removeChangeListener(ChangeListener r) {
        listener = SwingEventMulticaster.remove(listener, r);
    }

    public void setVariables(List variables) {
        if (variableList == variables)
            return;

        for (Iterator i = variableList.iterator(); i.hasNext();) {
            ((Variable) i.next()).removeChangeListener(this);
        }

        variableList = (variables != null) ? variables : Collections.EMPTY_LIST;
        variableTableModel.setVariables(variableList);

        for (Iterator i = variableList.iterator(); i.hasNext();) {
            ((Variable) i.next()).addChangeListener(this);
        }

        if (variableList.size() > 0)
            variableTable.setRowSelectionInterval(0, 0);

        revalidate();
        repaint();
    }

    // forward events from variables
    public void stateChanged(ChangeEvent event) {
        if (event instanceof VariableChangeEvent) {
            if (((VariableChangeEvent) event).isValue())
                fireStateChanged();
        } else {
            fireStateChanged();
        }
    }

    public void setFormat(ScientificFormat format) {
        variableTable.setFormat(format);
        slider.setFormat(format);
    }

    public Format getFormat() {
        return variableTable.getFormat();
    }

    private void fireStateChanged() {
        if (listener != null) {
            listener.stateChanged(new ChangeEvent(this));
        }
    }

    public void update() {
        setVariables(WiredPlugin.getPlugin().getVariables());
    }

  /** Called when this panel is selected or deselected. */
  public void setSelected(boolean selected) {
        setVariables(WiredPlugin.getPlugin().getVariables());
  }

}
