// Copyright 2004-2006, FreeHEP.
package hep.wired.heprep.projection;

import hep.wired.feature.Resetable;
import hep.wired.heprep.services.Projection;

/**
 * Shows azimuthal region in detail while displaying the full 360 degrees.
 * Formula: phi' = phi0 + PI + ((d*PI)*dPhi)/((d*PI)+PI-|dPhi|) where rho is
 * constant, dPhi = phi - phi0 - PI and d is between 0 and INF.
 * 
 * @author Mark Donszelmann
 * @version $Id: ClockProjection.java 8598 2006-08-14 20:38:02Z duns $
 */

public class ClockProjection extends VariableProjection implements Resetable {

    private static final double PI = Math.PI;

    private static final double PI2 = PI * 2;

    private double resetD, resetPhi0;

    private double d, phi0;

    public ClockProjection() {
        this(0.1, 0.1, 0, 0);
    }

    private ClockProjection(double d, double resetD, double phi0,
            double resetPhi0) {
        super("Clock");
        addVariable("d", 0.0, Double.MAX_VALUE, d, "rad",
                "Clock aperture angle");
        addVariable("phi0", -Double.MAX_VALUE, Double.MAX_VALUE, phi0, "rad",
                "Clock angle");
        this.resetD = resetD;
        this.resetPhi0 = resetPhi0;
    }

    public String getFormula() {
        return "(u,v,w) = ";
    }

    public double[] transform(double[] xyz) {
        double x = xyz[X];
        double y = xyz[Y];
        double z = xyz[Z];

        double phi = Math.atan2(y, x);
        double rho = Math.sqrt((x * x) + (y * y));

        double dPhi = phi - phi0 - PI;
        // make sure dPhi is in -PI..PI
        dPhi = (dPhi + PI + PI2) % PI2 - PI;

        double dPI = PI / d;

        double phiPrime = phi0 + PI + (dPI * dPhi)
                / (dPI + PI - Math.abs(dPhi));

        xyz[U] = rho * Math.cos(phiPrime);
        xyz[V] = rho * Math.sin(phiPrime);
        xyz[W] = z;

        return xyz;
    }

    public double[] deltaTransform(double[] xyz) {
        return transform(xyz);
    }

    public Projection copy() {
        return new ClockProjection(d, resetD, phi0, resetPhi0);
    }

    // Resetable
    public Object reset(Object newState) {
        Object oldState = new double[] { d, phi0 };
        if (newState == null) {
            d = resetD;
            phi0 = resetPhi0;
        } else {
            double[] array = (double[]) newState;
            d = array[0];
            phi0 = array[1];
        }
        return oldState;
    }

}
