// Copyright 2004-2006, FreeHEP.
package hep.wired.heprep.projection;

import hep.wired.feature.Resetable;
import hep.wired.heprep.services.InverseProjection;
import hep.wired.heprep.services.Projection;
import hep.wired.variable.DoubleVariable;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Blows up the center of the plot cylindrically while squeezing the outer
 * parts.
 * 
 * @author Mark Donszelmann
 * @version $Id: CylindricalFishEyeProjection.java 2161 2005-08-02 22:47:40Z
 *          duns $
 */

public class CylindricalFishEyeProjection extends VariableProjection implements
        InverseProjection, Resetable {

    private DoubleVariable varAlpha, varRhoMax, varZMax;

    private double alpha, resetAlpha;

    private double rhoMax, resetRhoMax;

    private double zMax, resetZMax;

    public CylindricalFishEyeProjection() {
        // FIXME: WIRED-320, we need the height and width
        this(0.01, 0.01, 6860, 4000, 6860, 4000);
    }

    private CylindricalFishEyeProjection(double alpha, double resetAlpha,
            double rhoMax, double resetRhoMax, double zMax, double resetZMax) {
        super("CylindricalFishEye");
        varAlpha = addVariable("alpha", 0.0, 0.03, alpha, null,
                "Distortion Factor");
        varRhoMax = addVariable("rhoMax", 0.1, 10000, rhoMax, null,
                "Maximum rho for distortion");
        varZMax = addVariable("zMax", 0.1, 10000, zMax, null,
                "Maximum Z for distortion");

        varRhoMax.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                setMaximumAlpha();
            }
        });

        varZMax.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                setMaximumAlpha();
            }
        });

        this.resetAlpha = resetAlpha;
        this.resetRhoMax = resetRhoMax;
        this.resetZMax = resetZMax;
    }

    private void setMaximumAlpha() {
        varAlpha.setMaximum(10 / Math.min(varRhoMax.getDoubleVariable(),
                varZMax.getDoubleVariable()));
    }

    public String getFormula() {
        return "(u,v,w) = " + "f(rho',phi), where rho'=rho/(1+a*rho), "
                + "f(rho',phi), where rho'=rho/(1+a*rho), " + "z / (1+a*|z|)";
    }

    public double[] transform(double[] xyz) {
        double x = xyz[X];
        double y = xyz[Y];
        double z = xyz[Z];

        double phi = Math.atan2(y, x);
        double rho = Math.sqrt((x * x) + (y * y));
        double rhoPrime = rho / (1 + (alpha * rho));
//        rhoPrime *= (1 + (alpha * Math.abs(rhoMax)));

        double zPrime = z / (1 + (alpha * Math.abs(z)));
//        zPrime *= (1 + (alpha * Math.abs(zMax)));

        xyz[U] = rhoPrime * Math.cos(phi);
        xyz[V] = rhoPrime * Math.sin(phi);
        xyz[W] = zPrime;

        return xyz;
    }

    public double[] deltaTransform(double[] xyz) {
        return transform(xyz);
    }

    public double[] inverseTransform(double[] uvw) throws UnsupportedOperationException {
        double u = uvw[U];
        double v = uvw[V];
        double w = uvw[W];
        
        double phi = Math.atan2(v, u);
        double rhoPrime = Math.sqrt((u * u) + (v * v));
        double rho = rhoPrime / (1 - (alpha * rhoPrime));
        double z = w / (1 - (alpha * Math.abs(w)));
        
        uvw[X] = rho * Math.cos(phi);
        uvw[Y] = rho * Math.sin(phi);
        uvw[Z] = z;
        return uvw;
    }

    public double[] inverseDeltaTransform(double[] uvw) throws UnsupportedOperationException {
        return inverseTransform(uvw);
    }

    public Projection copy() {
        return new CylindricalFishEyeProjection(alpha, resetAlpha, rhoMax,
                resetRhoMax, zMax, resetZMax);
    }

    // Resetable
    public Object reset(Object newState) {
        Object oldState = new double[] { alpha, rhoMax, zMax };
        if (newState == null) {
            alpha = resetAlpha;
            rhoMax = resetRhoMax;
            zMax = resetZMax;
        } else {
            double[] array = (double[]) newState;
            alpha = array[0];
            rhoMax = array[1];
            zMax = array[2];
        }
        return oldState;
    }
}
