// Copyright 2004-2005, FreeHEP.
package hep.wired.heprep.util;

import hep.wired.util.Matrix3D;
import hep.wired.util.UVWindices;
import hep.wired.util.XYZindices;

/**
 * Converts primitives into polypoints, for use in drawing as polylines or
 * polygons.
 * 
 * FIXME WIRED-270 NOTE: arcs could draw ELLIPSEPOINTS divided by the part of
 * the ellipse which they draw NOTE: linesOnArcs should be redone into faces
 * which can be filled.
 * 
 * @author Mark Donszelmann
 * @version $Id: PolyPoint.java 8598 2006-08-14 20:38:02Z duns $
 */

public class PolyPoint implements XYZindices, UVWindices {

    public static final int ELLIPSEPOINTS = 46;

    private PolyPoint() {
        // only static methods
    }

    /**
     * Converts a circle into a polypoint.
     */
    public static int circle(double r, double x, double y, double z,
            double phi, double theta, double[][] xyz) {
        return arc(r, r, x, y, z, phi, theta, 0, 0, Math.PI * 2, xyz);
    }

    /**
     * Converts an ellipse into a polypoint.
     */
    public static int ellipse(double rx, double ry, double x, double y,
            double z, double phi, double theta, double omega, double[][] xyz) {
        return arc(rx, ry, x, y, z, phi, theta, omega, 0, Math.PI * 2, xyz);
    }

    /**
     * Converts an arc into a polypoint.
     */
    public static int arc(double r, double x, double y, double z, double phi,
            double theta, double start, double end, double[][] xyz) {
        return arc(r, r, x, y, z, phi, theta, 0, start, end, xyz);
    }

    /**
     * Converts an arc into a polypoint.
     */
    public static int arc(double rx, double ry, double x, double y, double z,
            double phi, double theta, double omega, double start, double end,
            double[][] xyz) {

        int n = Math.min(xyz[X].length, ELLIPSEPOINTS);
        if (n <= 1)
            return 0;

        double delta = (end - start) / (n - 1);
        for (int i = 0; i < n; i++) {
            double angle = start + i * delta;
            xyz[X][i] = rx * Math.cos(angle);
            xyz[Y][i] = ry * Math.sin(angle);
            xyz[Z][i] = 0;
        }

        // FIXME WIRED-249 we still recreate a new Matrix here every time.
        Matrix3D m = new Matrix3D();
        m.rotate(omega, 0, 0, 1);
        m.rotate(theta, 0, 1, 0);
        m.rotate(phi, 0, 0, 1);
        m.translate(x, y, z);
        xyz = m.transform(xyz, n, false);

        return n;
    }

    /**
     * Converts an ellipsoid into three polypoints.
     */
    public static int ellipsoid(double rx, double ry, double rz, double x, double y,
            double z, double phi, double theta, double omega, double[][][] uvw) {

        int n = Math.min(uvw[X][U].length, ELLIPSEPOINTS);
        if (n <= 1)
            return 0;

        double delta = (2*Math.PI) / (n - 1);
        for (int i = 0; i < n; i++) {
            double angle = i * delta;
            uvw[X][U][i] = rx * Math.cos(angle);
            uvw[X][V][i] = ry * Math.sin(angle);
            uvw[X][W][i] = 0;
            
            uvw[Y][U][i] = rx * Math.cos(angle);
            uvw[Y][V][i] = 0;
            uvw[Y][W][i] = rz * Math.sin(angle);
            
            uvw[Z][U][i] = 0;            
            uvw[Z][V][i] = ry * Math.cos(angle);
            uvw[Z][W][i] = rz * Math.sin(angle);
        }

        // FIXME WIRED-249 we still recreate a new Matrix here every time.
        Matrix3D m = new Matrix3D();
        m.rotate(omega, 0, 0, 1);
        m.rotate(theta, 0, 1, 0);
        m.rotate(phi, 0, 0, 1);
        m.translate(x, y, z);
        
        uvw[X] = m.transform(uvw[X], n, false);
        uvw[Y] = m.transform(uvw[Y], n, false);
        uvw[Z] = m.transform(uvw[Z], n, false);

        return n;
    }

    
    /**
     * Converts the lines between two arcs into number of polypoints. Returns
     * number of lines generated. Each line is straight, containing start and
     * end point.
     */
    public static int linesOnArcs(double rx1, double ry1, double x1, double y1,
            double z1, double phi1, double theta1, double omega1, double rx2,
            double ry2, double x2, double y2, double z2, double phi2,
            double theta2, double omega2, double start, double end, int nLines,
            double[][][] xyz) {

        int n = Math.min(xyz.length, nLines);

        double delta = (end - start) / (n - 1);
        double uvw[] = new double[3];

        // FIXME WIRED-249 we still recreate a new Matrix here every time.
        Matrix3D m1 = new Matrix3D();
        m1.rotate(omega1, 0, 0, 1);
        m1.rotate(theta1, 0, 1, 0);
        m1.rotate(phi1, 0, 0, 1);
        m1.translate(x1, y1, z1);

        Matrix3D m2 = new Matrix3D();
        m2.rotate(omega2, 0, 0, 1);
        m2.rotate(theta2, 0, 1, 0);
        m2.rotate(phi2, 0, 0, 1);
        m2.translate(x2, y2, z2);
        for (int i = 0; i < n; i++) {
            double angle = start + i * delta;
            double cosAngle = Math.cos(angle);
            double sinAngle = Math.sin(angle);

            uvw[X] = rx1 * cosAngle;
            uvw[Y] = ry1 * sinAngle;
            uvw[Z] = 0;
            uvw = m1.transform(uvw, false);
            xyz[i][X][0] = uvw[X];
            xyz[i][Y][0] = uvw[Y];
            xyz[i][Z][0] = uvw[Z];

            uvw[X] = rx2 * cosAngle;
            uvw[Y] = ry2 * sinAngle;
            uvw[Z] = 0;
            uvw = m2.transform(uvw, false);
            xyz[i][X][1] = uvw[X];
            xyz[i][Y][1] = uvw[Y];
            xyz[i][Z][1] = uvw[Z];
        }

        return n;
    }
}
